
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <fcntl.h>
#include <time.h>
#include <unistd.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>

#define PIPE_OUT        0
#define PIPE_IN         1
#define EXEC_BUF        3000

#define DWORD   unsigned int
#define BYTE    unsigned char

struct  stat    st;

///////////////////////////////////////////////////////////////////////////////////////////
// Print file parameters from stat(st) structure into std output
//      Input:
//              st - global stat structure
void PrintSTAT()
{
	if (S_ISREG(st.st_mode))  printf("-");

	if (S_ISDIR(st.st_mode))  printf("d");

	if (S_ISCHR(st.st_mode))  printf("c");

	if (S_ISBLK(st.st_mode))  printf("b");

	if (S_ISFIFO(st.st_mode)) printf("p");

	if (S_ISLNK(st.st_mode))  printf("l");

	if (S_ISSOCK(st.st_mode)) printf("s");

	if (st.st_mode & S_IRUSR)  printf("r");
	else        printf("-");

	if (st.st_mode & S_IWUSR)  printf("w");
	else        printf("-");

	if (st.st_mode & S_IXUSR)  printf("x");
	else        printf("-");

	if (st.st_mode & S_IRGRP)  printf("r");
	else        printf("-");

	if (st.st_mode & S_IWGRP)  printf("w");
	else        printf("-");

	if (st.st_mode & S_IXGRP)  printf("x");
	else        printf("-");

	if (st.st_mode & S_IROTH)  printf("r");
	else        printf("-");

	if (st.st_mode & S_IWOTH)  printf("w");
	else        printf("-");

	if (st.st_mode & S_IXOTH)  printf("x");
	else        printf("-");

	printf(" %02d", st.st_nlink);
	printf(" %04d", st.st_uid);
	printf(" %04d", st.st_gid);
	printf("\t%4d", (DWORD)st.st_size);
	printf("\t%s", ctime(&st.st_mtime));

	return;
}
///////////////////////////////////////////////////////////////////////////////////////////
// Parse 'errno' defined in "errno.h"
// Print error description into std output
void PrintErr()
{
	if (errno & EACCES)
		printf("Search permission is denied for one of the directories in the path; ");

	if (errno & EBADF)
		printf("File is bad; ");

	if (errno & EFAULT)
		printf("Bad address; ");

	if (errno & ELOOP)
		printf("Too many symbolic links encountered while traversing the path; ");

	if (errno & ENAMETOOLONG)
		printf("File name too long; ");

	if (errno & ENOENT)
		printf("A component of the path does not exist; ");

	if (errno & ENOMEM)
		printf("Out of memory (i.e. kernel memory); ");

	if (errno & ENOTDIR)
		printf("A component of the path is not a directory; ");

	printf("\n");
	return;
}
///////////////////////////////////////////////////////////////////////////////////////////
// Executes file 'szFullName' with argv[] parameters by execv (in case byPMode == 1)
// or execvp (in case byPMode == 0) and translate it output into 'szRetString'
//      Return:
//              Zero if successful, otherwise -1.
int ShellExecute(char* szFullName, char** argv, char* szRetString, BYTE byPMode)
{
	int fd[2];          // File descriptors
	int nRet = 0;
	size_t size;

	if (pipe(fd) < 0)
	{
		printf("Can\'t create pipe\n");
		return -1;
	}

	nRet = fork();

	if (nRet == -1)
	{
		printf("Can\'t fork child\n");
		return -1;

	}
	else if (nRet > 0)
	{
		/* PARRENT */

		close(fd[PIPE_IN]);
		size = read(fd[PIPE_OUT], szRetString, EXEC_BUF);

		if (size < 0)
		{
			printf("Can\'t read pipe\n");
			return -1;
		}

		close(fd[PIPE_OUT]);

	}
	else
	{
		/* CHILD */
		close(fd[PIPE_OUT]);
		close(1);
		dup(fd[PIPE_IN]);
		close(fd[PIPE_IN]);

		if (byPMode)
			execvp(szFullName, argv);   // In case specified file name does not contain a slash (/) character
		else
			execv(szFullName, argv);   // Otherwise

		return -1; /* In case of exec error */
	}

	return 0;
};
///////////////////////////////////////////////////////////////////////////////////////////
// Collect information about needed files and print into std output
//  Return:
//              Zero when succeeded
int Search4Files(void)
{
	struct      dirent* dp;     // Next directory entry
	DIR* dir;           // Directory descriptor

	printf("------------------------------------------------------------------------------------------------\n");
	printf("File Name\t\t\t\tProtection  L  USR  GRP\t Size\tLast modification date\n");
	printf("------------------------------------------------------------------------------------------------\n");

	printf("/etc/udev/rules.d/95-grdnt.rules\t");

	if (stat("/etc/udev/rules.d/95-grdnt.rules", &st) != -1)
		PrintSTAT();
	else
		PrintErr();

	printf("/etc/udev/rules.d/95-grdnt_hid.rules\t");

	if (stat("/etc/udev/rules.d/95-grdnt_hid.rules", &st) != -1)
		PrintSTAT();
	else
		PrintErr();

	printf("/etc/hotplug/usb/grdnt.usermap\t\t");

	if (stat("/etc/hotplug/usb/grdnt.usermap", &st) != -1)
		PrintSTAT();
	else
		PrintErr();

	printf("/etc/hotplug/usb/grdnt\t\t\t");

	if (stat("/etc/hotplug/usb/grdnt", &st) != -1)
		PrintSTAT();
	else
		PrintErr();

	printf("------------------------------------------------------------------------------------------------\n");

	if ((dir = opendir("/etc/udev/rules.d")) == NULL)
	{
		printf("\n## Cannot open \"/etc/udev/rules.d\" directory.\n");
		return 1;
	}

	while ((dp = readdir(dir)) != NULL)
	{
		if (strstr(dp->d_name, "grdnt")
				&& strcmp(dp->d_name, "95-grdnt.rules")
				&& strcmp(dp->d_name, "95-grdnt_hid.rules")
				&& strcmp(dp->d_name, "95-grdnt.rules\r")
				&& strcmp(dp->d_name, "95-grdnt_hid.rules\r"))
			printf("Also found:\t/etc/udev/rules.d/%s\n", dp->d_name);
	}

	closedir(dir);

	return 0;
};
///////////////////////////////////////////////////////////////////////////////////////////
// Collect some system info and print into std output
void GetSysInfo(void)
{
	char szRetString[EXEC_BUF]; // String returned by executed function
	char* szFullName = NULL;    // Name of function to execute
	char* argv[5] = {NULL, NULL, NULL, NULL, NULL};             // Arguments
	char* p1, *p2;                      // Auxiliary pointer
	int fd;             // File descriptor
	int i;              // Cycle counter

	fd = open("/proc/version", 0);

	if (fd != -1)
	{
		read(fd, szRetString, EXEC_BUF);

		if (strstr(szRetString, " (gcc"))
			*strstr(szRetString, " (gcc") = 0x0A;

		printf("%s", szRetString);
		close(fd);
	}
	else
	{
		printf("/proc/version\t\t\t");
		PrintErr();
	}

	szFullName = "lspci";
	memset(szRetString, 0, EXEC_BUF);
	argv[0] = "lspci";
	ShellExecute(szFullName, argv, szRetString, 1);

	if (strstr(szRetString, "bridge"))
	{
		p1 = strstr(szRetString, "bridge");

		if (strstr(p1, "\n"))
		{
			p2 = strstr(p1, "\n");
			*(p2 + 1) = 0;
		}

		printf("%s", szRetString);
	}
	else
	{
		fd = open("/proc/bus/pci/00/00.0", 0);

		if (fd != -1)
		{
			memset(szRetString, 0, EXEC_BUF);
			read(fd, szRetString, EXEC_BUF);
			printf("/proc/bus/pci/00/00.0 :\n");

			for (i = 00; i < 32; i++)
				printf("%02X ", (BYTE) * (szRetString + i));

			printf("\n");

			for (i = 32; i < 64; i++)
				printf("%02X ", (BYTE) * (szRetString + i));

			printf("\n");
			close(fd);
		}
		else
		{
			printf("/proc/version\t\t\t");
			PrintErr();
		}
	}

	fd = open("/proc/cpuinfo", 0);

	if (fd != -1)
	{
		read(fd, szRetString, EXEC_BUF);

		if (strstr(szRetString, "model name"))
		{
			p1 = strstr(szRetString, "model name");

			if (strstr(p1, "\n"))
			{
				p2 = strstr(p1, "\n");
				*(p2 + 1) = 0;
			}
		}

		printf("%s", szRetString);
		close(fd);
	}
	else
	{
		printf("/proc/version\t\t\t");
		PrintErr();
	}

	return;
};
///////////////////////////////////////////////////////////////////////////////////////////
// main
int main()
{
	printf("\nWelcome to Guardant sysinfo tool!\n");

	printf("\nSystem info ...\n");
	GetSysInfo();

	printf("\nSearching for files ...\n");
	Search4Files();

	return 0;
}
